"""

    Mobile site testing utils
   
   These are in separate file, so that third party products can use them easily.
"""

__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

def setDiscriminateMode(request, mode):
    """ Spoof the discrimination mode. 
    
    Poke HTTP request internals to look like it is directed
    to domain name based discriminating.
    
    @param mode: 'mobile' or 'web'
    """
    
    def setURL(url):
        request.other["URL"] = url
        request.other["ACTUAL_URL"] = url        
        request.other["SERVER_URL"] = url        
        
    if mode == "mobile":
        host = "mobile.nohost"
    elif mode == "preview":
        host = "preview.nohost"        
    elif mode == "web":
        host = "web.nohost"
    else:
        raise RuntimeError("Unknown mode:" + mode)
    
    setURL("http://" + host)
    request.environ["HTTP_HOST"] = host
    



"""

    Helper middleware for Plone mobile edition.
    
    TODO: UNDONE

    
"""

__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

from paste.request import get_cookies, parse_headers

class SubdomainCookieSharingMiddleware:
    """ WSGI compliant middleware for setting cookie domain info.
    
    By default, Plone cookies are not shared with subdomains.
    Since mobile site and preview page rendering use subdomains,
    hitting a preview page requires reauth.
    
    This middleware works around the problem by setting the domain header
    of the cookie.
    """
    def __init__(self, app=None):
        self.app = app
            
    def __call__(self, environ, start_response):
        # TODO: WAS BAD IDEA
        pass
   

def subdomain_middleware_factory(app, global_conf, **local_conf):
    """ Paste Deploy script compliant middleware constructor.
    
    http://pythonpaste.org/deploy/#paste-filter-factory
    """
    return SubdomainCookieSharingMiddleware(app)


                



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

## Controller Python Script ""
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=path
##title=Initial post-login actions
##

portal_url = context.portal_url()
view = context.restrictedTraverse("@@mobile_image_resizer")
uri = view.getResizedImageURL(path)
return uri



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

from zope.component import getUtility, queryUtility

from Products.CMFCore.Skinnable import SkinnableObjectManager
from Products.CMFCore.utils import getToolByName

from interfaces import IMobileUtility

# Apply monkeypatches
import monkeypatch

def initialize(context):
    """Initializer called when used as a Zope 2 product."""



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

import urlparse

import zope.interface

from Products.Five.browser import BrowserView

from zope.component import getUtility, queryUtility

from gomobile.mobile.interfaces import IMobileUtility

class MobileUtility(object):
    """ Zope 3 utility for mobile actions. """
    
    zope.interface.implements(IMobileUtility)
    
    def isMobileRequest(self, request):
        """ Determine should this request be rendered in mobile mode. """        
            
        if request.get("view_mode") == "mobile":
            # IFRAME request
            return True
        
        if "HTTP_HOST" in request.environ:
            # Unit tests don't have this
            if "mobi." in request.environ["HTTP_HOST"]:
                return True
        
        if self.isPreviewRequest(request):
            return True
                
        return False
    
    def isPreviewRequest(self, request):
        """ Determine should this request be rendered in mobile mode. """        

        if "HTTP_HOST" in request.environ:
            if "preview." in request.environ["HTTP_HOST"]:
                return True

        if request.get("view_mode") == "mobile":
            # IFRAME request
            return True
                
        return False    
    
       
class MobileTool(BrowserView):
    """ Traversable mobile utility. """
    
    def getUtility(self):        
        return getUtility(IMobileUtility)
    
    def replaceNetworkLocation(self, url, newNetLocation):
        parts = urlparse.urlparse(url)
        parts = list(parts)
        parts[1] = newNetLocation
        return urlparse.urlunparse(parts)
    
    def isMobileRequest(self):
        return self.getUtility().isMobileRequest(self.request)        
    
    def isPreviewRequest(self):
        return self.getUtility().isPreviewRequest(self.request)            
    
    def getMobileSiteURL(self):
        """ Return the mobile version of this of """
        base = self.context.portal_properties.mobile_properties.mobile_site_base
        return self.replaceNetworkLocation(self.context.absolute_url(), base)
    
    def getMobilePreviewURL(self):
        """ Return URL used in phone simualtor.
        
        We do here little ugly magic, because IFRAME forces
        us to use same domain for parent and child document, or
        cross frame JS does not work.
        """
        base = self.context.portal_properties.mobile_properties.preview_site_base
        return self.replaceNetworkLocation(self.context.absolute_url(), base)
        
    def getWebSiteURL(self):
        """ Return the mobile version of this of """
        base = self.context.portal_properties.mobile_properties.web_site_base
        return self.replaceNetworkLocation(self.context.absolute_url(), base)
    



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

import urlparse

import zope.interface

from gomobile.mobile.interfaces import IMobileRequestDiscriminator, MobileRequestType, IMobileSiteLocationManager
from zope.app.component.hooks import getSite

class DomainNameBasedMobileSiteLocationManager(object):
    """ Present the content differently based on its domain name.
    
    xxx.com -> web site
    mobi.xxx.com -> mobile site
    preview.xxx.com -> mobile site preview
    
    There transforms here are port neutral i.e.
    the port of the orignal HTTP request is preserved in the URL.
    
    Use portal_properties.mobile_properties domains names 
    to change the content media. URI part stays same for all content.    
    """
    
    zope.interface.implements(IMobileSiteLocationManager)
                    
    def _getBaseDomainName(self, domain, properties):
        """        
        @param domain: domain name as string
        @param properties: Mobile site config (see propertiestool.xml)
        @return: domain name without mobile or preview prefixes
        """
        parts = domain.split(".")
        
        all_subdomain_prefixes = properties.mobile_domain_prefixes + \
                                 properties.preview_domain_prefixes +  \
                                 properties.web_domain_prefixes        
        
        # Chop prefix subdomain
        for p in all_subdomain_prefixes:
            if parts[0] == p:
                parts = parts[1:]
                break
            
        return ".".join(parts)
    
    def _prefixDomain(self, domain, mode, properties):
        """ Add subdomain discriminator to domain host name
        
        xxx.com -> m.xxx.com
        
        @param mode: MobileRequestType
        @param domain: domain name as string
        @param properties: Mobile site config (see propertiestool.xml)
        @return: mangled domain name
        
        """
        if mode == MobileRequestType.MOBILE:
            # Use first prefix on the list when rewriting
            return properties.mobile_domain_prefixes[0] + "." + domain
        elif mode == MobileRequestType.PREVIEW:
            # Use first prefix on the list when rewriting
            return properties.preview_domain_prefixes[0] + "." + domain
        else:
            # Assume web domains shouldn't get prefixed
            return domain
            
    def _replaceNetworkLocation(self, url, mode, properties):
        """ Rewrite domain name in the site URL.
        
        @param url: url to be rewritten
        @param mode: One of MobileRequestType pseudo constants
        @param properties: Mobile site config (see propertiestool.xml)
        """
        parts = urlparse.urlparse(url)
        parts = list(parts)
        
        # domain name + port as tuple
        domainAndPort = parts[1].split(":")
        
        base = self._getBaseDomainName(domainAndPort[0], properties)
        prefixed =  self._prefixDomain(base, mode, properties)
        
        domainAndPort[0] = prefixed
                
        parts[1] = ":".join(domainAndPort)
        
        return urlparse.urlunparse(parts)
    
    def rewriteURL(self, request, url, mode):
        
        # Load PloneSite object from thread locals
        site = getSite()
        
        # Load config from the database
        properties = site.portal_properties.mobile_properties
        
        return self._replaceNetworkLocation(url, mode, properties)
    
    
    def intercept(self, site, request):
        pass
        
            



"""
    Python monket patched code to make twinapex.plone.mobile work properly.
    
"""
from base64 import encodestring, decodestring
from urllib import quote, unquote
import urlparse
import logging

from zope.component import getUtility, queryUtility
from zope.component.interfaces import ComponentLookupError

from Products.CMFCore.Skinnable import SkinnableObjectManager
from Products.CMFCore.utils import getToolByName

from interfaces import MobileRequestType, IMobileRequestDiscriminator

logger = logging.getLogger("Plone")
#
# Monkey patch in the mobile support
#
# TODO do something smarter here
#
def getSkinNameFromRequest(self, REQUEST=None):
    '''Returns the skin name from the Request.'''

    if REQUEST is None:
        return None
    sfn = self.getSkinsFolderName()
    
    if sfn is not None:
       
        sf = getattr(self, sfn, None)
        
        properties = self.portal_properties

        try:
            discriminator = getUtility(IMobileRequestDiscriminator)
        except ComponentLookupError:
            # This happens aaaalways....
            raise RuntimeError("Cannot load IMobileRequestDiscriminator utility. This usually means that there has been error in your ZCML or Python modules imported by ZCML. Please see Zope start up logs.")
        
        site = self        

        # Determine mobile skin name from mobile_properties
        modes = discriminator.discriminate(site, REQUEST)
        
        if MobileRequestType.MOBILE in modes:
        
            if hasattr(sf, "mobile_base") and hasattr(properties, "mobile_properties"):
                # Plone Mobile quickinstaller has been run
                # and we have mobile theme specifc files registeed
                
                mobile_properties = properties.mobile_properties
                                                                
                # Enable mobile specific skin layer
                skin_name = mobile_properties.mobile_skin
                return skin_name
        
        if sf is not None:
            return REQUEST.get(sf.getRequestVarname(), None)

SkinnableObjectManager.getSkinNameFromRequest = getSkinNameFromRequest


# Monkey patch authentication cookie (__ac)
# to support subdomains - otherwise mobile page and preview page
# requests require re auth
# Ugly but can't do otherwise
from Products.PluggableAuthService.plugins.CookieAuthHelper import CookieAuthHelper

def getCookieDomain(request):
    """ Get a parent domain name to be used in the cookie 
    
    One cookie covers all *.yoursite.com domains.
    """

    if "SERVER_URL" in request.environ:
        # WSGI based servers
        server_url = request.environ["SERVER_URL"]
    else:
        # Zope's Medusa
        server_url = request.other["SERVER_URL"]
    
    parts = urlparse.urlparse(server_url)
    netloc = parts[1]
    domain = netloc.split(":")[0]
    
    # Looks like at least Firefox does not allow set cookie domain ".localhost"
    # ugh
    # things gets ugly here
    
    # web.localhost -> localhost
    if "localhost" in domain:
        logger.warn("Mobilized Plone site must be configured to be accessed using subdomains.")
        logger.warn("Otherwise preview and mobile site access.")
        logger.warn("Add following to your /etc/hosts:")
        logger.warn("127.0.0.1 mobi.site.foo web.site.foo preview.site.foo")
        logger.warn("..and access site site using fake domain name web.site.foo.")
        logger.warn("Also DO NOT do Zope HTTP Basic Auth, since it is not subdomain aware - use only Plone interface for login")        
        logger.warn("See twinapex.plone.mobile.monkeypatch for more info")
    
    if len(domain.split(".")) == 1:
        raise RuntimeError("You need to use subdomain to access the site. E.g. web.localhost instead of localhost")
    
    domain_root = ".".join(domain.split(".")[1:])
    
    return "." + domain_rot


def updateCredentials(self, request, response, login, new_password):
    """ Respond to change of credentials (NOOP for basic auth). """
    
    
    cookie_str = '%s:%s' % (login.encode('hex'), new_password.encode('hex'))
    cookie_val = encodestring(cookie_str)
    cookie_val = cookie_val.rstrip()
    
    domain = getCookieDomain(request)
    
    response.setCookie(self.cookie_name, quote(cookie_val), path='/', domain=domain)

def resetCredentials(self, request, response):
    """ Raise unauthorized to tell browser to clear credentials. """
    domain = getCookieDomain(request)
    response.expireCookie(self.cookie_name, path='/', domain=domain)

CookieAuthHelper.resetCredentials = resetCredentials




__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

from Products.Five import zcml
from Products.Five import fiveconfigure

from Testing import ZopeTestCase as ztc

from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup

import utils

@onsetup
def setup_zcml():

    fiveconfigure.debug_mode = True
    import gomobile.convergence
    zcml.load_config('configure.zcml', gomobile.convergence)
    fiveconfigure.debug_mode = False
    
    # We need to tell the testing framework that these products
    # should be available. This can't happen until after we have loaded
    # the ZCML.

    ztc.installPackage('gomobile.mobile')    
    #ztc.installPackage('gomobile.convergence')

    
    
# The order here is important.
setup_zcml()
ptc.setupPloneSite(products=['gomobile.mobile'])

class BaseTestCase(ptc.PloneTestCase):
    """We use this base class for all the tests in this package. If necessary,
    we can put common utility or setup code in here.
    """
    
    def setUp(self):
        ptc.PloneTestCase.setUp(self)     
        
    def setDiscriminateMode(self, mode):
        utils.setDiscriminateMode(self.portal.REQUEST, mode)



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

import random

def importFinalSteps(context):
    """
    The last bit of code that runs as part of this setup profile.
    """
    site = context.getSite()
    
    # Reseed mobile image resizer secret
    site.portal_properties.mobile_properties.image_resizer_secret = str(random.randint(0, 999999999))



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

import os, sys
import unittest
from gomobile.mobile.tests.base import BaseTestCase

from Products.CMFCore.utils import getToolByName

from gomobile.mobile.browser import resizer


class TestMobileImages(BaseTestCase):
    
    def afterSetUp(self):
        self.workflow = getToolByName(self.portal, 'portal_workflow')
        self.acl_users = getToolByName(self.portal, 'acl_users')
        self.types = getToolByName(self.portal, 'portal_types')
        
        # Reload Plone skins
        self._refreshSkinData()
        
        # Upload an AT field based image
        self.loginAsPortalOwner()
        self.portal.invokeFactory("Image", "test_img")
        
        path = os.path.dirname(sys.modules[__name__].__file__)
        path = os.path.join(path, "logo.jpg")
        
        f = open(path)
        #f.filename  = path
        portal = self.portal        
        self.portal.test_img.setImage(f)
        f.close()
        self.logout()        
        
    def runResize(self, img):
        """
        
        @param img: Traversing path to the image
        """
        request = self.portal.REQUEST
        query = resizer.getResizedImageQuery(path=img, width=10, height=10)
        
        for key, val in query.items():
            request.set(key, val)            
                
        view = self.portal.restrictedTraverse("@@mobile_image_resizer")    
        
        output = view()
        
                
    def test_res_info(self):
        """ Check that we can extract width/height from resource based images """
        img  = "++resource++gomobile.mobile/phone_back.png" 
        portal = self.portal
        img_obj = portal.unrestrictedTraverse(img)
        
        
    def test_skin_info(self):
        """ Check that we can extract width/height from Zope skin layer based images """        
        portal = self.portal
        img = "logo.jpg"
        img_obj = portal.unrestrictedTraverse(img)
        
    
    def test_res_resizer(self):        
        img  = "++resource++gomobile.mobile/phone_back.png" 
        self.runResize(img)
    
    def test_skin_resizer(self):        
        img  = "logo.jpg" 
        self.runResize(img)

    def test_at_resizer(self):        
        img  = "test_img/getImage" 
        self.runResize(img)
        
            
    
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestMobileImages))
    return suite



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

from Acquisition import aq_inner

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.layout.viewlets.common import ViewletBase
from zope.component import getMultiAdapter

class ConvergedViewletMixin(object):
    """ Viewlet which is aware whether it is in web or mobile
        and does not render itself in the wrong environment.
        
    """
    def getTool(self):
        return self.context.unrestrictedTraverse("mobile_tool")

    def isOnCorrectMedia(self):
        raise NotImplementedError("Subclass must implement")
    
    def render(self):
        """ Conditionally render the portlet only if we are on a correct media. """
        if self.isOnCorrectMedia():
            return ViewletBase.render(self)
        else:
            return ""
    
    def getCanvasScaledImage(self, *args, **kwargs):
        """ Return image src which will go through image resizer to make image fit to the mobile screen.
        
        """        

        resizer = self.context.restrictedTraverse("@@mobile_image_resizer")
        return resizer.getResizedImageURL(*args, **kwargs)
    
    def getCanvaScaledBackgroundImage(self, *args, **kwargs):
        """ Return style attribute to scale background image """
        src = self.getCanvasScaledImage(*args, **kwargs)
        return "background-image: url(" + src + ")"

       
class WebOnlyViewletMixin(ConvergedViewletMixin):
    """ Viewlet which is in a shared mobile/web viewlet manager, but should be rendered in web only. """

    def isOnCorrectMedia(self):
        return not self.getTool().isMobileRequest()

class MobileOnlyViewletMixin(ConvergedViewletMixin):
    """ Viewlet which is in a shared mobile/web viewlet manager, but should be rendered in mobile only. """
    
    def isOnCorrectMedia(self):
        return self.getTool().isMobileRequest()
    


class MobileFolderListingViewlet(MobileOnlyViewletMixin, ViewletBase):
    """ Displays the contents of the current folder. 
    
    We don't have navigation here, so we need to emulate it.
    
    TODO: This class must be scrapped and rewritten.
    """
    
    index = ViewPageTemplateFile('mobile_folder_listing.pt')
    
    
    def getLanguage(self):
        """ LinguaPlone compatible language getter """
        
        if not "LANGUAGE" in self.request.other:
            # We are called in before traverse hook
            # LanguageTool language bindings have not been run yet...
            # run it
            self.context.portal_languages.getLanguageBindings()
        
        lang = self.request.other["LANGUAGE"]
        
        return lang
    

    def update(self):
        """ TODO: rewrite using enhanced navtree functions now when we have them """
        
        self.items = []
                
        
        if not self.isOnCorrectMedia():
            # Do not to heavy calculations for web view
           return
            
        context_state = getMultiAdapter((self.context, self.request),
                                        name=u'plone_context_state')

        context = aq_inner(self.context)
        
        
        from Products.CMFCore.interfaces._content import IFolderish
        if not IFolderish.providedBy(context):
            # Default page - use folder above
            bads = [context]
            context = context.aq_parent
        else:
            bads = []
                                
        bad_types = context.portal_properties.navtree_properties.metaTypesNotToList
        bad_types = list(bad_types)
        #bad_types.append("Image")
        #bad_types.append("ATImage")
        
        # Test if this item can control its listing
        # TODO:Use interfaces here
        if hasattr(context, "Schema"):    
            schema = context.Schema()
            if "mobileFolderListing" in schema:
                field = schema["mobileFolderListing"]
                value = field.get(context)
                
                # Listing prohibited
                if value != True:
                    self.items = []
                    return
        
        if hasattr(context, "default_page"):    
            default_page = context.default_page
        else:
            default_page = None
        
        def is_excluded(subobject):
        
            if subobject == default_page:
                return True
                                
            if hasattr(subobject, "getExcludeFromNav"):
                return subobject.getExcludeFromNav()
            return False

        items = context.listFolderContents()
        items = [ i for i in items if not i in bads ]        
        items = [ i for i in items if not i.meta_type in bad_types ]
        items = [ i for i in items if not is_excluded(i) ]
        
        # Filter by language
        language = self.getLanguage()
        if language:
            
            def item_lang(x):
                if hasattr(x, "getLanguage"):
                    return i.getLanguage()
                else:
                    return None
            
            items = [ i for i in items if item_lang(i) == language ]
        
        # TODO: Fix this depedecncy somehow!
        # Check listed o
        #from gomobile.convergence.tools import checkMediaFilter, filterObject
        #from gomobile.convergence.interfaces import ContentMediaOption
        #strategy = ContentMediaOption.MOBILE
            
        #items = [ i for i in items if filterObject(obj, strategy) ]

        # Now check which items have navigation button images
        self.items = []
        for item in items:
            image = None

            # gomobile.convergence extender hooks in here
            field = item.Schema().getField("mobileButtonImage")
            if field:
                value = field.get(item)
                if value:
                    image = item.absolute_url() + "/mobileButtonImage"
                
            self.items.append({ "item" : item, "image": image})
            
                        
from plone.app.layout.viewlets import common
class LogoViewlet(MobileOnlyViewletMixin, common.LogoViewlet):    
    """ Mobile site logo """
    index = ViewPageTemplateFile('logo.pt')
    
    def update(self):
        common.LogoViewlet.update(self)
        self.portal = self.portal_state.portal()        
        logoName = "logo.jpg"        

class FooterViewlet(MobileOnlyViewletMixin, common.ViewletBase):    
    """ Mobile site footer """
    index = ViewPageTemplateFile('footer.pt')

    def update(self):
        """ Make sure that we have all view template context variables set up """
        
        common.ViewletBase.update(self)

        # Get tabs (top level navigation links)
        context_state = getMultiAdapter((self.context, self.request),
                                    name=u'plone_context_state')
        actions = context_state.actions()

        
        portal_tabs_view = getMultiAdapter((self.context, self.request),
                                       name='portal_tabs_view')
    
        
        self.portal_tabs = portal_tabs_view.topLevelTabs(actions=actions)

    
class PathBarViewlet(MobileOnlyViewletMixin, common.PathBarViewlet):    
    """ Mobile site breadcrumbs """
    index = ViewPageTemplateFile('path_bar.pt')
    
           
from plone.app.layout.analytics.view import AnalyticsViewlet
class MobileAwareAnalyticsViewlet(WebOnlyViewletMixin, AnalyticsViewlet):
    """ Do not render Google Analytics in mobile """

    def render(self):
        """ Conditionally render the portlet only if we are on a correct media. """
        if self.isOnCorrectMedia():
            return AnalyticsViewlet.render(self)
        else:
            return ""
    



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

import urllib

from zope.app.component.hooks import getSite
import zope.interface

from zope.interface import implements
from zope.component import getMultiAdapter, getUtility
from zope.app.container.interfaces import INameChooser
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Acquisition import aq_base, aq_inner, aq_parent
from OFS.SimpleItem import SimpleItem
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from Products.CMFPlone.browser import ploneview
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility, queryUtility
from zope.app.component.hooks import getSite


from gomobile.mobile.interfaces import IMobileUtility, IMobileRequestDiscriminator, IMobileSiteLocationManager, MobileRequestType
       

class MobileSimulator(BrowserView):
    """ Render mobile site preview in phone simulator view.
    
    """

    context = None
    request = None
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.membership = context.portal_membership
        
    def __call__(self):
        pass
    
class MobileTool(BrowserView):
    """ A context-aware wrapper for mobile site utilities.
    
    Provide convience functions for page templates.
    """
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.discriminator = getUtility(IMobileRequestDiscriminator)
        self.location_manager = getUtility(IMobileSiteLocationManager)
                
        self.request_flags = self.discriminator.discriminate(self.context, self.request)
                
    def isMobileRequest(self):
        return MobileRequestType.MOBILE in self.request_flags
    
    def isPreviewRequest(self):
        return MobileRequestType.PREVIEW in self.request_flags
    
    def isWebRequest(self):
        return MobileRequestType.WEB in self.request_flags
    
    def isAdminRequest(self):
        return MobileRequestType.ADMIN in self.request_flags
         
    def getMobileSiteURL(self):
        """ Return the mobile version of this context"""        
        return self.location_manager.rewriteURL(self.request, self.context.absolute_url(), MobileRequestType.MOBILE)
    
    def getMobilePreviewURL(self):
        """ Return URL used in phone simualtor.            
        """
        return self.location_manager.rewriteURL(self.request, self.context.absolute_url(), MobileRequestType.PREVIEW)
        
    def getWebSiteURL(self):
        """ Return the web version URL of this of context """        
        return self.location_manager.rewriteURL(self.request, self.context.absolute_url(), MobileRequestType.WEB)
    
    




"""

    Code for determining whether requests are mobile or web.

"""

__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

import zope.interface

from gomobile.mobile.interfaces import IMobileRequestDiscriminator, MobileRequestType

from zope.app.component.hooks import getSite

class DefaultMobileRequestDiscriminator(object):
    """ Determine which are web, mobile, mobile preview and mobile content admin HTTP requests.
    
    1. Determine preview and mobile requests based on a domain name.
       (m.xxx, xxx.mobi, mobi.xxx, mobile.xxx are considered to be mobile).
    
    2. Use HTTP GET query parameter view_mode to override the default behavior
       (for dummy testing)
       
    """
    
    zope.interface.implements(IMobileRequestDiscriminator)
    
    def __init__(self):
        pass
        
    
    def isMobileRequest(self, site, request, mobileDomainPrefixes):
        """ Determine should this request be rendered in mobile mode. """        

        #
        # Special HTTP GET parameter tellign
        # that render this as a preview request
        #            
        if request.get("view_mode") == "mobile":
            # IFRAME request
            return True


        #
        # Determine if this is targeted to mobile domain
        # based on request domain name
        #
        if "HTTP_HOST" in request.environ:
            
            host = request.environ["HTTP_HOST"]
            
                    
            for prefix in mobileDomainPrefixes:
                if host.startswith(prefix + "."):
                    return True

                                    
        return False
    
    def isPreviewRequest(self, site, request, prefixes):
        """ Determine should this request be rendered in mobile mode. 
        
        
        """        

        #
        # If we are using preview. domain name
        # this is a preview request
        #
        
        if "HTTP_HOST" in request.environ:
            for prefix in prefixes:
                if request.environ["HTTP_HOST"].startswith(prefix):
                    return True

        #
        # Special HTTP GET parameter tellign
        # that render this as a preview request
        #
        if request.get("view_mode") == "mobile":
            # IFRAME request
            return True
                
        return False    

    def isAdminRequest(self, site, request):    
        """ By default, assume all logged in users are admins.
        """ 
        return not site.portal_membership.isAnonymousUser()
    
    def discriminate(self, context, request):
        
        flags = []
                
        # Load settings from database
        try:
            properties = context.portal_properties.mobile_properties
            
            # It is possible to have several mobile domain prefixes,
            # but in the default config there is just one
            mobileDomainPrefixes = properties.mobile_domain_prefixes      
            previewDomainPrefixes = properties.preview_domain_prefixes
            
        except AttributeError:
            # Traversing happens when our product is not properly set up
            # or during the site launch. The loading order prevents 
            # us to access mobile_properties here, so it is 
            # safe to assume this was not a mobile request
            
            # (Because monkey-patch intercepts admin interface requests also
            # we need to make sure no exceptio gets through)
            flags.append(MobileRequestType.WEB)
            return flags
        

        # 1. Test against preview subdomains
        if self.isPreviewRequest(context, request, previewDomainPrefixes):
            flags.append(MobileRequestType.PREVIEW)
            flags.append(MobileRequestType.MOBILE)
        # 2. Test against mobile subdomains
        elif self.isMobileRequest(context, request, mobileDomainPrefixes):
            flags.append(MobileRequestType.MOBILE)    
        # 3. otherwise assume web request
        else:
            flags.append(MobileRequestType.WEB)

            # 4. show mobile management portlets based on 
            # if the user has logged in
            if self.isAdminRequest(context, request):
                flags.append(MobileRequestType.ADMIN)
        
            
        return flags



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

import md5
import urllib
from StringIO import StringIO

from AccessControl import Unauthorized
from zope.app.component.hooks import getSite
import zope.interface

from zope.interface import implements
from zope.component import getMultiAdapter, getUtility
from zope.app.container.interfaces import INameChooser
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from Products.CMFPlone.browser import ploneview
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility, queryUtility
from zope.app.component.hooks import getSite


from gomobile.mobile.interfaces import IMobileUtility, IMobileRequestDiscriminator, IMobileSiteLocationManager, MobileRequestType
from gomobile.imageinfo.interfaces import IImageInfoUtility
# Resizer     
_secret = None

# To not exceed this resize dimensions
safe_width = 1000
safe_height = 1000

def getSecret():
    """ Avoid properties look up using cached value """
    global _secret
    
    if not _secret:
        site = getSite()
        _secret = site.portal_properties.mobile_properties.image_resizer_secret
        
    return _secret
        
              
def calculateSignature(**kwargs):
    """ Calculate protected MD5 for resizing parameters, so that input is protected against DoS attack """

    for key, value in kwargs.items():
        concat = key + "=" + str(value)
    concat += getSecret()
    return md5.new(concat).hexdigest()


def getResizedImageQuery(**params):
    """ Convert parameters to HTTP GET query form and calculate protection signature for them """
    
    for key, val in params.items():
        params[key] = str(val)
    
    secret = calculateSignature(**params)
    
    params["secret"] = secret
    
    return params

def getResizedImageURL(*args, **kwargs):
    """ Get image URI which is resized version for the mobile screen.

    @param path: Zope traversing path to the image
    
    @param width: Target width or None for to use the mobile screen width
    
    @param width: Target height or None for to use the mobile screen height
    """
    
    if(len(args) > 0):
        raise RuntimeError("No nameless arguments allowed:" + str(args))
    
    site = getSite()
    query = getResizedImageQuery(**kwargs)
    return site.absolute_url() + "/@@mobile_image_resizer?" + urllib.urlencode(query)

class ImageResizerView(BrowserView):
    """ Resize images on fly for mobile screens.
    
    Respond to the request with image/png content.
    
    GET query parameters tell the wanted image dimensions. Query parameters
    are protected with md5 generated from a shared secret to prevent DoS 
    attacks. The image is identified by parameter 'path' which
    is Zope traversing path to the image.
    
    Automatic width or height parameter can be used. In this case
    we see whether we have sniffed the mobile screen size based
    on user agent sniffer middle. Use mobile browser canvas
    dimension in this case.
    
    If width/height is automatic, but no browser information is availabl,e
    fallback to default setting set in mobile_properties.    
    """
    
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def getResizedImageURL(self, *args, **kwargs):
        return getResizedImageURL(*args, **kwargs)

    def __call__(self):
        """ 
        """        
        params = {}
        params["path"] = self.request["path"]
        params["width"] = self.request["width"]
        params["height"] = self.request["height"]


        # Override parameter signature check
        # by knowing the shared secret
        override_secret = self.request.get("override_secret", None)

        if "padding_width" in self.request:
            padding_width = int(self.request["padding_width"])
        else:
            padding_width = 0

        params["conserve_aspect_ration"] = self.request.get("conserve_aspect_ration", None)

        
        secret = self.request["secret"]
        
        if override_secret:
            # Override parameter signature check
            # by directly providing shared secret as an HTTP query parameter
            # for testing.
            if override_secret != getSecret():
                raise Unauthorized("Wrong override_secret:" + override_secret)        
        else:
            if calculateSignature(**params) != secret:
                raise Unauthorized("Bad image resizer secret:" + str(secret) + " " + str(params))        
            

        if "browser" in self.request.environ:
            browser = self.request.environ["browser"]
            
            # TODO: Make something smarter here
            if browser.is_generic() or "Firefox" in browser.user_agent:
                # "Firefox" in browser.user_agent <-- what the shit is in the database??? firefox.canvas_width = 350
                browser = None
            
        else:
            browser = None
            
            
        print "Resizer got browser:" + str(browser)
        
       
        # Solve wanted width
        if params["width"] == "auto":
            
            # twinapex.sniffer middleware inserts this
                
            if browser:
                width = browser.canvas_width    
                
                # TODO: Hardcoded hack
                if width > 320:
                    width = 320                                            
            else:
                width = 0
                
            if width <= 1:
                print "Using default canvas width"
                width = self.context.portal_properties.mobile_properties.default_canvas_width
                
        else:
            width = int(params["width"])
            
        width -= padding_width

        # Solve wanted height
        if params["height"] == "auto":
            
            # twinapex.sniffer middleware inserts this
                
            if browser:
                height = browser.canvas_height
            else:
                height = 0
                
            if height <= 1:
                print "Using default canvas height"
                height = self.context.portal_properties.mobile_properties.default_canvas_height
                
        else:
            height = int(params["height"])
       
        if width < 1 or width > safe_width:
            raise Unauthorized("Invalid width: %d" % width)
        
        if height < 1 or height > safe_height:
            raise Unauthorized("Invalid height: %d" % height)        
        
        tool = getUtility(IImageInfoUtility)
                
        # Convert from string to Python bool
        if params["conserve_aspect_ration"]:
            conserve_aspect_ration = (params["conserve_aspect_ration"] != "False")
        else:
            conserve_aspect_ration = True
            
        print "Resizing %d %d" % (width, height)
                
        data, format = tool.getResizedImage(params["path"], width, height, conserve_aspect_ration=conserve_aspect_ration)
        
        self.request.response.setHeader("Content-type", "image/" + format)
        
        if isinstance(data, StringIO):
            # Looks like ZMedusa server cannot stream data to the client...
            return data.getvalue()
        
        return data
        
        
        
                                         
        



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

import zope.interface
from plone.theme.interfaces import IDefaultPloneLayer
from zope.viewlet.interfaces import IViewletManager

class IMobileLayer(IDefaultPloneLayer):
    """Marker interface that defines a Zope 3 browser layer.
    
    This layer is applied on HTTPRequest when mobile rendering 
    is on.    
    """
    
class IMobileUtility(zope.interface.Interface):
    """ Helper function to deal with mobile requests. """
    
class MobileRequestType:
    """ Pseudoconstant flags how HTTP request can relate to mobility """
    
    # Admin web page
    ADMIN = "admin"
    
    # Anonymous web page
    WEB = "web"
    
    # Mobile page
    MOBILE = "mobile"
    
    # Preview mobile page
    PREVIEW = "preview"
    
    

class IMobileRequestDiscriminator(zope.interface.Interface):
    """ Determine if the request is a mobile or not. 
        
    """

    def isMobileRequest(context, request):
        """ Is the HTTP request a mobile version.
        
        If the request is mobile request
        
            - Mobile skin layer will be applied
            
            - Navigation will be mobile based
            
        @param context: PloneSite object
        
        @param request: HTTPRequest
        
        @return: True if the request output is targeted to mobile site.
        """
        
    def isPreviewRequest(context, request):        
        """
        
        Preview requests are mobile pages rendered in IFRAME.
        They need special handling, because they load simulator
        specific Javascript files.

        @param context: PloneSite object

        @param request: HTTPRequest
        
        @return: True if the request output is targeted to preview IFRAME        

        """
        
    def isAdminRequest(context, request):    
        """ Is the request mobile site administration request.
        
        The admin request will render the site in normal web mode,
        but mobile only content is visibile in folder listings
        and navigation tree. For the simple fact, that 
        you need to access mobile content to be able to edit it.

        @param context: PloneSite object
        
        @param request: HTTPRequest
        
        @return: True if the request output should render mobile only specific content on 
                 normal web view
        """
        
    def discriminate(context, request):
        """ Flag request to describe its relation of mobile
        
        Wrapper which calls everyone of above methods.    
        
        This function just exist to make your life easier.
        
        The result is allowed to be cached internally.
        
        @return list of strings of MobileRequestType flags
        """
        
class IMobileSiteLocationManager(zope.interface.Interface):
    """ Use cookie based switching for web/mobile mode rendering.

    1. Set necessary cookies which tell whether the site should be rendered
      in web or mobile mode
    
    2. Redirect to different URL
    """
    
    
      
    
    def rewriteURL(request, url, mode):
        """ Rewrite the URL to redirect to the page in a different mobile view mode.
        
        @param mode: One of MobileRequestType pseudo constants
        
        @return: string
        """
        
      



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"

from zope.interface import implements
from zope.component import getUtility
from zope import schema

from plone.app.portlets.portlets import base
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.portlets.interfaces import IPortletDataProvider

from zope.component import getUtility, queryUtility
from gomobile.mobile.interfaces import IMobileUtility

class IPreviewPortlet(IPortletDataProvider):
    """ Define buttons for mobile preview """
    pass

class Renderer(base.Renderer):
    """ Overrides static.pt in the rendering of the portlet. """
    
    index = ViewPageTemplateFile('preview.pt')
    
    def __init__(self, context, request, view, manager, data):
        base.Renderer.__init__(self, context, request, view, manager, data)    
        self.site_url = getToolByName(context, 'portal_url')
        
    def getMobileSiteURL(self):
        """ """
        mobile_tool = self.context.unrestrictedTraverse("mobile_tool")
        return mobile_tool.getMobileSiteURL()

    def getMobilePreviewURL(self):
        """ """
        mobile_tool = self.context.unrestrictedTraverse("mobile_tool")
        return mobile_tool.getMobilePreviewURL()
      
    def is_visible(self):
      membership_tool=self.context.portal_membership
      return not membership_tool.isAnonymousUser()      
      
    def render(self):
        if self.is_visible():
          return self.index()
        else:
          return ""

class Assignment(base.Assignment):
    """ Assigner for grey static portlet. """
    implements(IPreviewPortlet)
    
    title = "Mobile preview"    
    
class AddForm(base.NullAddForm):
    """ Make sure that add form creates instances of our custom portlet instead of the base class portlet. """
    def create(self):
        return Assignment()    



__license__ = "GPL 2.1"
__copyright__ = "2009 Twinapex Research"


import unittest

from zope.component import getUtility, queryUtility

from Products.CMFCore.utils import getToolByName

from gomobile.mobile.tests.base import BaseTestCase
from gomobile.mobile.interfaces import IMobileUtility, IMobileRequestDiscriminator, IMobileSiteLocationManager, MobileRequestType

class TestDiscriminator(BaseTestCase):
    
    def afterSetUp(self):
        self.workflow = getToolByName(self.portal, 'portal_workflow')
        self.acl_users = getToolByName(self.portal, 'acl_users')
        self.types = getToolByName(self.portal, 'portal_types')
        
        # Reload Plone skins
        self._refreshSkinData()
        
    def test_is_mobile_request(self):
        """ Check that we can extract width/height from resource based images """
        
        # By default, we are in web mode
        self.logout()
        
        util = getUtility(IMobileRequestDiscriminator)
        flags = util.discriminate(self.portal, self.portal.REQUEST)
                
        self.assertEqual(len(flags), 1)
        self.assertTrue("web" in flags)
        
        self.setDiscriminateMode("mobile")
        util = getUtility(IMobileRequestDiscriminator)
        flags = util.discriminate(self.portal, self.portal.REQUEST)
        self.assertEqual(len(flags), 1)
        self.assertTrue("mobile" in flags)
        
        self.setDiscriminateMode("preview")
        util = getUtility(IMobileRequestDiscriminator)
        flags = util.discriminate(self.portal, self.portal.REQUEST)
        self.assertEqual(len(flags), 2)
        self.assertTrue("mobile" in flags)
        self.assertTrue("preview" in flags)
        
        self.setDiscriminateMode("web")
        self.loginAsPortalOwner()
        util = getUtility(IMobileRequestDiscriminator)
        flags = util.discriminate(self.portal, self.portal.REQUEST)
        self.assertEqual(len(flags), 2)
        self.assertTrue("web" in flags)
        self.assertTrue("admin" in flags)
        
        
    def test_mobile_tool_traverse(self):
        tool = self.portal.unrestrictedTraverse("@@mobile_tool")
        self.assertFalse(tool.isMobileRequest())
        
        # Retraverse since we poke HTTPRequest
        self.setDiscriminateMode("mobile")
        tool = self.portal.unrestrictedTraverse("@@mobile_tool")
        self.assertTrue(tool.isMobileRequest())
        
    def test_rewrite_url(self):
        """ See that we can present the content URL in different media modes """
        tool = self.portal.unrestrictedTraverse("@@mobile_tool")
        
        preview = tool.getMobilePreviewURL()

        self.assertTrue("preview." in preview)

        mobile = tool.getMobileSiteURL()
        self.assertTrue("m." in mobile)
        
        web = tool.getWebSiteURL()
        self.assertTrue(web.startswith("http://nohost"))
        
    
    def test_skin_switch(self):
        """ See that correct skin is seleted based on request """
        
        
        # TODO: Code this test
        # very difficult to do since, 
        # unit tests treat skins differently
        return
        
        from Products.CMFCore import Skinnable
                        
        
        # Per-request per thread active skins
        SKINDATA = Skinnable.SKINDATA
        import pdb ; pdb.set_trace()
        
        # Assume we have only one thread
        data = SKINDATA.items()[0]
        
        self.setDiscriminateMode("mobile")
        
    
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestDiscriminator))
    return suite
